@@ -1,27 +1,34 @@
 package Maypole::Model::Base;
-our %remember;
-sub MODIFY_CODE_ATTRIBUTES { $remember{$_[1]} = $_[2]; () }
 
-sub FETCH_CODE_ATTRIBUTES { $remember{$_[1]} } 
-sub view :Exported { }
-sub edit :Exported { }
+use Maypole::Constants;
+use attributes ();
+
+our %remember;
+
+sub MODIFY_CODE_ATTRIBUTES { $remember{ $_[1] } = $_[2]; () }
+
+sub FETCH_CODE_ATTRIBUTES { $remember{ $_[1] } }
 
 sub process {
-    my ($class, $r) = @_;
+    my ( $class, $r ) = @_;
     my $method = $r->action;
-    return if $r->{template}; # Authentication has set this, we're done.
+    return if $r->{template};    # Authentication has set this, we're done.
 
     $r->{template} = $method;
-    $r->objects([]);
+    $r->objects( [] );
     my $obj = $class->retrieve( $r->{args}->[0] );
     if ($obj) {
-        $r->objects([ $obj ]);
-        shift @{$r->{args}};
+        $r->objects( [$obj] );
+        shift @{ $r->{args} };
     }
-    $class->$method($r, $obj, @{$r->{args}});
+    $class->$method( $r, $obj, @{ $r->{args} } );
 }
 
-sub display_columns { 
+sub list_columns {
+    shift->display_columns;
+}
+
+sub display_columns {
     sort shift->columns;
 }
 
@@ -31,21 +38,17 @@ Maypole::Model::Base - Base class for model classes
 
 =head1 DESCRIPTION
 
+This is the base class for Maypole data models. This is an abstract class
+meant to define the interface, and can't be used directly.
+
+=head2 process
+
+This is the engine of this module. It populates all the relevant variables
+and calls the requested action.
+
 Anyone subclassing this for a different database abstraction mechanism
 needs to provide the following methods:
 
-=head2 do_edit
-
-If there is an object in C<$r-E<gt>objects>, then it should be edited
-with the parameters in C<$r-E<gt>params>; otherwise, a new object should
-be created with those parameters, and put back into C<$r-E<gt>objects>.
-The template should be changed to C<view>, or C<edit> if there were any
-errors. A hash of errors will be passed to the template.
-
-=cut
-
-sub do_edit { die "This is an abstract method" }
-
 =head2 setup_database
 
     $model->setup_database($config, $namespace, @data)
@@ -74,39 +77,66 @@ This turns an ID into an object of the appropriate class.
 This is called on an model class representing a table and allows the
 master model class to do any set-up required. 
 
-=head2 related
-
-This can go either in the master model class or in the individual
-classes, and returns a list of has-many accessors. A brewery has many
-beers, so C<BeerDB::Brewery> needs to return C<beers>.
-
 =head2 columns
 
 This is a list of all the columns in a table. You may also override
-C<display_columns>, which is the list of columns you want to view, in
-the right order.
+see also C<display_columns>
 
 =head2 table
 
 This is the name of the table.
 
+=cut 
+
+sub class_of       { die "This is an abstract method" }
+sub setup_database { die "This is an abstract method" }
+
 =head2 Commands
 
 =over
 
+=item do_edit
+
+If there is an object in C<$r-E<gt>objects>, then it should be edited
+with the parameters in C<$r-E<gt>params>; otherwise, a new object should
+be created with those parameters, and put back into C<$r-E<gt>objects>.
+The template should be changed to C<view>, or C<edit> if there were any
+errors. A hash of errors will be passed to the template.
+
+=cut
+
+sub do_edit { die "This is an abstract method" }
+
+
 =item list
 
 The C<list> method should fill C<< $r-> objects >> with all of the
 objects in the class. You may want to page this using C<Data::Page> or
 similar.
 
+=item edit
+
+Empty Action
+
+=item view
+
+Empty Action.
+
+
 =back
 
 =cut
 
-sub class_of       { die "This is an abstract method" }
-sub setup_database { die "This is an abstract method" }
-sub list :Exported { die "This is an abstract method" };
+
+sub list : Exported {
+    die "This is an abstract method";
+}
+
+sub view : Exported {
+}
+
+sub edit : Exported {
+}
 
 =pod
 
@@ -117,16 +147,30 @@ Also, see the exported commands in C<Maypole::Model::CDBI>.
 Additionally, individual derived model classes may want to override the
 following methods:
 
+=head2 display_columns
+
+Returns a list of columns to display in the model. by default returns
+all columns in alphabetical order. Override this in base classes to
+change ordering, or elect not to show columns.
+
+=head2 list_columns
+
+Same as display_columns, only for listings. Defaults to display_columns
+
 =head2 column_names
 
 Return a hash mapping column names with human-readable equivalents.
 
 =cut
 
-sub column_names { my $class = shift; map { 
+sub column_names {
+    my $class = shift;
+    map {
         my $col = $_;
         $col =~ s/_+(\w)?/ \U$1/g;
-        $_ => ucfirst $col } $class->columns }
+        $_ => ucfirst $col
+    } $class->columns;
+}
 
 =head2 description
 
@@ -136,5 +180,34 @@ A description of the class to be passed to the template.
 
 sub description { "A poorly defined class" }
 
-1;
+=head2 is_public
 
+should return true if a certain action is supported, or false otherwise. 
+Defaults to checking if the sub has the :Exported attribute.
+
+=cut
+
+sub is_public {
+    my ( $self, $action ) = @_;
+    my $cv = $self->can($action);
+    return 0 unless $cv;
+    my $attrs = join " ", attributes::get($cv);
+    do {
+        warn "$action not exported" if Maypole->debug;
+        return 0;
+    } unless $attrs =~ /\bExported\b/i;
+    return 1;
+}
+
+=head2 related
+
+This can go either in the master model class or in the individual
+classes, and returns a list of has-many accessors. A brewery has many
+beers, so C<BeerDB::Brewery> needs to return C<beers>.
+
+=cut
+
+sub related {
+}
+
+1;
